<?php

namespace Drupal\entity_bundle_scaffold\Commands;

/**
 * Trait providing common validators for other commands.
 *
 * @property EntityTypeManagerInterface $entityTypeManager
 */
trait ValidatorsTrait {

  /**
   * Validate whether an entity type exists.
   */
  public function validateEntityType(string $entityTypeId): void {
    if (!$this->entityTypeManager->hasDefinition($entityTypeId)) {
      throw new \InvalidArgumentException(
            t("Entity type with id ':entityType' does not exist.", [':entityType' => $entityTypeId])
        );
    }
  }

  /**
   * Validate whether an entity type exists and is an ECK entity type.
   */
  public function validateEckEntityType(string $entityTypeId): void {
    $this->validateEntityType($entityTypeId);

    $definition = $this->entityTypeManager->getDefinition($entityTypeId);

    if ($definition->getProvider() !== 'eck') {
      throw new \InvalidArgumentException(
            t("Entity type with id ':entityType' is not an ECK entity type.", [':entityType' => $entityTypeId])
        );
    }
  }

}
