<?php

namespace Drupal\entity_bundle_scaffold;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\entity_bundle_scaffold\Service\Helper\EntityBundleClassMethodGeneratorHelper;
use PhpParser\Builder\Method;
use PhpParser\BuilderFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for entity bundle class method generator plugins.
 */
abstract class EntityBundleClassMethodGeneratorBase extends PluginBase implements EntityBundleClassMethodGeneratorInterface, ContainerFactoryPluginInterface {

  /**
   * The entity bundle class method generator helper service.
   *
   * @var \Drupal\entity_bundle_scaffold\Service\Helper\EntityBundleClassMethodGeneratorHelper
   */
  protected $helper;

  /**
   * A factory for PHP parser builders.
   *
   * @var \PhpParser\BuilderFactory
   */
  protected $builderFactory;

  /**
   * Constructs an EntityBundleClassMethodGeneratorBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\entity_bundle_scaffold\Service\Helper\EntityBundleClassMethodGeneratorHelper $helper
   *   The entity bundle class method generator helper service.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array $plugin_definition,
    EntityBundleClassMethodGeneratorHelper $helper
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->helper = $helper;
    $this->builderFactory = new BuilderFactory();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_bundle_scaffold.entity_bundle_class_method_generator.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  abstract public function buildGetter(FieldDefinitionInterface $field, Method $method, array &$uses): void;

  /**
   * {@inheritdoc}
   */
  public function buildSetter(): void {
    // @todo Implement field setters
  }

}
